from bs4 import BeautifulSoup
from requests import get
from pathlib import Path
from .lib import read_data, write_data, check_columns


def create_parser(subparsers):
	prepare_parser = subparsers.add_parser('prepare', description="Parse source csv file from https://memopzk.org")
	prepare_parser.add_argument("-i", "--input", required=True, help="input csv file with data", type=Path)

	group = prepare_parser.add_mutually_exclusive_group(required=True)
	group.add_argument("-y", "--replace", action="store_true", help="replace input csv file")
	group.add_argument("-o", "--output", help="output csv file", type=Path)

	prepare_parser.add_argument("-d", "--delimiter", help="delimiter in the input csv file")
	prepare_parser.add_argument("-c", "--encoding", help="encoding of the input csv file", default="UTF-8")
	prepare_parser.add_argument("-v", "--verbosity", action="count", default=0, help="add progress messages")

	group = prepare_parser.add_mutually_exclusive_group()
	group.add_argument("-q", "--quiet", action="store_true", help="ignore errors")
	group.add_argument("-e", "--error", help="log file for errors")

	return subparsers


def get_description_and_img(src: str):
	soup = BeautifulSoup(get(src).content, 'html.parser')
	description_tag = soup.select_one('div.human-dossier__art > p > p')
	if not description_tag or description_tag.text.strip().startswith("Включение конкретного человека в список"):
		description_tag = soup.select_one('div.human-dossier__art > p')
		assert description_tag
	
	description = description_tag.text.strip().replace("""
""", "").replace("""\r""", " ").replace("  ", " ").replace("  ", " ")
	img = soup.select_one('div.human-dossier-card__img > img').get("src")
	return description, img


def parse_raw_data(data: list, args):
	width = len(str(len(data)))
	for i, row in enumerate(data, 1):
		try:
			row["city"] = ""
			row["description"], row["img_src"] = get_description_and_img(row["Полная информация"])
			if args.verbosity > 0 and i % 10 == 0 or args.verbosity > 1:
				print(f"Done: {i:>{width}d}/{len(data)}")
		except AssertionError:
			row["city"], row["description"], row["img_src"] = "", "", ""
			if not args.quiet:
				print(*row, file=args.error)


def prepare(args):
	"""Preparing a data file from https://memopzk.org site for filling in.
	
	The source CSV file must contain columns "ФИО" and "Полная информация".
	The output CSV file will contain filled columns "description", "img_src" and an empty column "city"."""
	if args.replace:
		args.output = args.input
	data = read_data(args)
	if not data or not check_columns(data[0].keys(), "ФИО", "Полная информация"):
		return
	parse_raw_data(data, args)
	write_data(data, args)





from argparse import Namespace
from xml.dom.minidom import Document
from pathlib import Path
from .lib import read_data, check_columns


def create_parser(subparsers):
	convert_parser = subparsers.add_parser('convert', description="Convertor data from CSV to KML")
	convert_parser.add_argument("-i", "--input", required=True, help="input csv file with data", type=Path)
	convert_parser.add_argument("-o", "--output", help="output kml file", type=Path)
	convert_parser.add_argument("-d", "--delimiter", help="delimiter in the input csv file", default=",")
	convert_parser.add_argument("-c", "--encoding", help="encoding of the input csv file", default="UTF-8")
	convert_parser.add_argument("-s", "--suffix", help="the phrase added to the description of each person")
	convert_parser.add_argument("-l", "--layout", help="the internal name of the xml document and the name of the layer on the map")
	return subparsers


def create_placemark(row: dict, src_name: str):
	placemark_tag = Document().createElement("Placemark")

	name_tag = Document().createElement("name")
	name_tag.appendChild(Document().createTextNode(row["ФИО"]))
	placemark_tag.appendChild(name_tag)

	p_tag = Document().createElement("p")
	p_tag.appendChild(Document().createTextNode(row["description"] + src_name))
	description_tag = Document().createElement("description")
	if "img_src" in row and row["img_src"]:
		img_tag = Document().createElement("img")
		img_tag.setAttribute("src", row["img_src"])
		img_tag.setAttribute("alt", f"""Фотография: {row["ФИО"]}""")
		description_tag.appendChild(Document().createCDATASection(img_tag.toprettyxml() + p_tag.toprettyxml()))
	else:
		description_tag.appendChild(Document().createCDATASection(p_tag.toprettyxml()))

	placemark_tag.appendChild(description_tag)

	point_tag = Document().createElement("Point")
	coordinates_tag = Document().createElement("coordinates")
	coordinates_tag.appendChild(Document().createTextNode(f"{row['lon']}, {row['lat']}, 0"))
	point_tag.appendChild(coordinates_tag)
	placemark_tag.appendChild(point_tag)
	
	return placemark_tag


def convert_data_to_kml(data: list, layout: str, suffix: str) -> Document:
	root = Document().createElement("kml")
	root.setAttribute("xmlns", "http://www.opengis.net/kml/2.2")
	doc_tag = Document().createElement("Document")
	name_tag = Document().createElement("name")
	name_tag.appendChild(Document().createTextNode(layout))
	doc_tag.appendChild(name_tag)

	for row in data:
		doc_tag.appendChild(create_placemark(row, suffix))
	root.appendChild(doc_tag)
	return root


def convert(args: Namespace) -> None:
	"""Convert a datafile from CSV to KML format.
	
	The source file must contain columns "ФИО", "description", "lon", and "lat"."""
	if args.output is None:
		args.output = args.input.with_suffix(".kml")
	if args.suffix is None:
		args.suffix = ""
	else:
		args.suffix = " " + args.suffix.strip()
	if args.layout is None:
		args.layout = args.input.stem
	
	data = read_data(args)
	if not data or not check_columns(data[0].keys(), "ФИО", "description", "lon", "lat"):
		return
	print(convert_data_to_kml(data, args.layout, args.suffix).toprettyxml(), file=open(args.output, "w", encoding=args.encoding))





from .convert import convert, create_parser as create_convert_parser
from .geocode import geocode, check, create_parser as create_geocode_parser
from .prepare import prepare, create_parser as create_prepare_parser
from .lib import create_parser


def main():
	parser = create_parser()
	subparsers = parser.add_subparsers(dest='command')
	create_prepare_parser(create_geocode_parser(create_convert_parser(subparsers)))
	args = parser.parse_args()
	commands = {"check": check, "prepare": prepare, "geocode": geocode, "convert": convert}
	if args.command not in commands:
		parser.print_help()
	else:
		commands[args.command](args)


main()



from collections.abc import Iterable, Mapping
from argparse import ArgumentParser, Namespace
from csv import DictReader, DictWriter, Sniffer
from importlib.metadata import version


def read_data(args: Namespace) -> Iterable[Mapping]:
	with open(args.input, newline='', encoding=args.encoding) as csvfile:
		if args.delimiter is None:
			dialect = Sniffer().sniff(csvfile.read(1024))
			csvfile.seek(0)
			args.delimiter = dialect.delimiter
		return list(DictReader(csvfile, delimiter=args.delimiter))


def write_data(data: Iterable[Mapping], args: Namespace) -> None:
	with open(args.output, 'w', newline='', encoding=args.encoding) as csvfile:
		writer = DictWriter(csvfile, fieldnames=data[0].keys())
		writer.writeheader()
		for row in data:
			writer.writerow(row)


def check_columns(headings: Iterable[str], *col_names) -> bool:
	for col_name in col_names:
		if col_name not in headings:
			print(f"""Error: Missing column "{col_name}"!""")
			print(f"Found columns: {headings}")
			print("Check used CSV delimiter!")
			return False
	return True


def create_parser() -> ArgumentParser:
	parser = ArgumentParser(description="Miltitool for convert data from https://memopzk.org to Google Maps")
	parser.add_argument("-v", "--version", action='version', version=version("kgr"))
	return parser


